package com.javaxing.account.service.impl;

import io.seata.core.context.RootContext;
import io.seata.rm.tcc.api.BusinessActionContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import com.javaxing.account.entity.AccountTbl;
import com.javaxing.account.mapper.AccountTblMapper;
import com.javaxing.account.service.AccountTblService;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;

@Service
@Slf4j
public class AccountTblServiceImpl implements AccountTblService{

    @Resource
    private AccountTblMapper accountTblMapper;

    @Override
    @Transactional
    public void deduct(String userId, BigDecimal money) {
        log.info("当前 XID: {}", RootContext.getXID());

        // 判断余额是否足够扣减
        AccountTbl accountTbl = accountTblMapper.findFirstByUserId(userId);
        if (money.compareTo(accountTbl.getMoney()) == 1) {
            // 超出余额，抛出异常
            throw new RuntimeException("余额不足");
        }

        log.info("余额扣减成功，扣减余额：{}，用户ID：{}", money, accountTbl.getUserId());
        // 开始扣减余额
        accountTblMapper.deductBalance(userId, money);
        log.info("余额扣减成功，剩余余额：{}，用户ID：{}", accountTbl.getMoney().subtract(money), accountTbl.getUserId());
    }

    @Override
    public boolean commit(BusinessActionContext actionContext) {
        log.info("commit成功");
        return true;
    }

    @Override
    public boolean rollback(BusinessActionContext actionContext) {
        // 回滚余额
        String userId = actionContext.getActionContext("userId").toString();
        BigDecimal money = new BigDecimal(actionContext.getActionContext("money").toString());

        accountTblMapper.addBalance(userId, money);
        log.info("余额回滚成功");
        return true;
    }


}
