package com.javaxing.order.service.impl;

import com.javaxing.order.entity.OrderTbl;
import com.javaxing.order.feign.AccountFeignService;
import com.javaxing.order.feign.StockFeignService;
import com.javaxing.order.service.BussinessService;
import com.javaxing.order.service.OrderTblService;
import com.javaxing.order.vo.OrderVO;
import io.seata.core.context.RootContext;
import io.seata.spring.annotation.GlobalTransactional;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


@Service
@Slf4j
public class BusinessServiceImpl implements BussinessService {

    @Autowired
    private OrderTblService orderTblService;
    @Autowired
    private AccountFeignService accountFeignService;
    @Autowired
    private StockFeignService stockFeignService;

    /**
     * 创建订单
     * @param vo
     */
    @Override
    @GlobalTransactional(name="addOrder",rollbackFor=Exception.class)
    public OrderTbl addOrder(OrderVO vo) {
        log.info("=============用户下单=================");
        log.info("当前 XID: {}", RootContext.getXID());
        // 创建订单
        //获取全局唯一订单号  测试使用
        Long orderId = System.currentTimeMillis();
        OrderTbl order = orderTblService.prepareSaveOrder(vo,orderId);
        // 扣减库存
        stockFeignService.deduct(vo.getCommodityCode(),vo.getCount());
        // 扣减余额
        accountFeignService.deduct(vo.getUserId(),vo.getMoney());
        return order;
    }
}
