package com.javaxing.order.service.impl;
import java.util.List;

import com.javaxing.order.entity.OrderStatus;
import com.javaxing.order.feign.AccountFeignService;
import com.javaxing.order.feign.StockFeignService;
import com.javaxing.order.vo.OrderVO;
import io.seata.rm.tcc.api.BusinessActionContext;
import io.seata.rm.tcc.api.BusinessActionContextParameter;
import io.seata.spring.annotation.GlobalTransactional;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import com.javaxing.order.entity.OrderTbl;
import com.javaxing.order.mapper.OrderTblMapper;
import com.javaxing.order.service.OrderTblService;
import org.springframework.transaction.annotation.Transactional;

@Service
@Slf4j
public class OrderTblServiceImpl implements OrderTblService{

    @Resource
    private OrderTblMapper orderTblMapper;

    /**
     * 新增订单记录
     * @param vo
     * @param orderId
     * @return
     */
    @Override
    @Transactional(rollbackFor=Exception.class)
    public OrderTbl prepareSaveOrder(OrderVO vo, @BusinessActionContextParameter(paramName = "orderId") Long orderId) {
        OrderTbl orderTbl = new OrderTbl();
        orderTbl.setId(orderId);
        orderTbl.setUserId(vo.getUserId());
        orderTbl.setCount(vo.getCount());
        orderTbl.setMoney(vo.getMoney());
        orderTbl.setStatus(OrderStatus.INIT.getValue());
        orderTbl.setCommodityCode(vo.getCommodityCode());
        int i = orderTblMapper.insertSelective(orderTbl);

        log.info("保存订单{}", i > 0 ? "成功" : "失败");
        return orderTbl;
    }

    @Override
    public boolean commit(BusinessActionContext actionContext) {
        orderTblMapper.updateStatusById(OrderStatus.SUCCESS.getValue(),Long.parseLong(actionContext.getActionContext("orderId").toString()));
        log.info("commit成功，订单状态为成功");
        return true;
    }

    @Override
    public boolean rollback(BusinessActionContext actionContext) {
        orderTblMapper.updateStatusById(OrderStatus.FAIL.getValue(),Long.parseLong(actionContext.getActionContext("orderId").toString()));
        log.info("回滚成功，订单状态为失败");
        return true;
    }






}
